Exception: Shells::NotRunning

Inherits:
ShellError
  • Object
show all
Defined in:
lib/shells/errors.rb

Overview

An error raised when a method requiring a running shell is called when the shell is not currently running.