Exception: Shells::QuitNow

Inherits:
Exception
  • Object
show all
Defined in:
lib/shells/errors.rb

Overview

Raise a QuitNow to tell the shell to stop processing and exit.

This is intentionally NOT a RuntimeError since it should only be caught inside the session.